@@ -110,6 +110,9 @@ class D(object):
 
         self.analytes = set(analytes)
 
+        # set for recording calculated ratios
+        self.analyte_ratios = set()
+
         # calculate total counts
         self.data['total_counts'] = sum(self.data['rawdata'].values())
 
@@ -149,7 +152,8 @@ class D(object):
         self.sigrng = np.array([]).reshape(0, 2)
 
         # set up filtering environment
-        self.filt = filt(self.Time.size, self.analytes)
+        # self.filt = filt(self.Time.size, self.analytes)
+        self.filt = None
 
         if errorhunt:
             print('   -> OK')
@@ -161,6 +165,9 @@ class D(object):
             a = self.analytes
         return sorted(a, key=analyte_sort_fn)
 
+    def _init_filts(self, analytes):
+        self.filt = filt(self.Time.size, analytes)
+
     @_log
     def setfocus(self, focus):
         """
@@ -497,16 +504,22 @@ class D(object):
             analytes = self.analytes
         elif isinstance(analytes, str):
             analytes = [analytes]
-
-        self.data['ratios'] = Bunch()
+        
+        if 'ratios' not in self.data.keys():
+            self.data['ratios'] = Bunch()
         for a in analytes:
-            self.data['ratios'][a] = (self.data['bkgsub'][a] /
+            if a == internal_standard:
+                continue
+            analyte_ratio = f'{a}_{self.internal_standard}'
+            self.data['ratios'][analyte_ratio] = (self.data['bkgsub'][a] /
                                       self.data['bkgsub'][self.internal_standard])
+            self.analyte_ratios.update([analyte_ratio])
+            self.cmap[analyte_ratio] = self.cmap[a]
         self.setfocus('ratios')
         return
 
     @_log
-    def calibrate(self, calib_ps, analytes=None):
+    def calibrate(self, calib_ps, analyte_ratios=None):
         """
         Apply calibration to data.
 
@@ -523,13 +536,13 @@ class D(object):
         None
         """
         # can have calibration function stored in self and pass *coefs?
-        if analytes is None:
-            analytes = self.analytes
+        if analyte_ratios is None:
+            analyte_ratios = self.analyte_ratios
 
         if 'calibrated' not in self.data.keys():
             self.data['calibrated'] = Bunch()
 
-        for a in analytes:
+        for a in analyte_ratios:
             m = calib_ps[a]['m'].new(self.uTime)
 
             if 'c' in calib_ps[a]:
@@ -539,9 +552,8 @@ class D(object):
 
             self.data['calibrated'][a] = self.data['ratios'][a] * m + c
 
-        if self.internal_standard not in analytes:
-            self.data['calibrated'][self.internal_standard] = \
-                np.empty(len(self.data['ratios'][self.internal_standard]))
+        # initialise filtering framework
+        self._init_filts(self.analyte_ratios)
 
         self.setfocus('calibrated')
         return
@@ -686,7 +698,7 @@ class D(object):
                         params, setn=setn)
 
     @_log
-    def filter_gradient_threshold(self, analyte, win, threshold, recalc=True):
+    def filter_gradient_threshold(self, analyte, win, threshold, recalc=True, win_mode='mid', win_exclude_outside=True, absolute_gradient=True):
         """
         Apply gradient threshold filter.
 
@@ -710,6 +722,17 @@ class D(object):
             Window used to calculate gradients (n points)
         recalc : bool
             Whether or not to re-calculate the gradients.
+        win_mode : str
+            Whether the rolling window should be centered on the left, middle or centre
+            of the returned value. Can be 'left', 'mid' or 'right'.
+        win_exclude_outside : bool
+            If True, regions at the start and end where the gradient cannot be calculated
+            (depending on win_mode setting) will be excluded by the filter.
+        absolute_gradient : bool
+            If True, the filter is applied to the absolute gradient (i.e. always positive),
+            allowing the selection of 'flat' vs 'steep' regions regardless of slope direction.
+            If Falose, the sign of the gradient matters, allowing the selection of positive or
+            negative slopes only.
 
         Returns
         -------
@@ -720,11 +743,14 @@ class D(object):
 
         # calculate absolute gradient
         if recalc or not self.grads_calced:
-            self.grads = calc_grads(self.Time, self.focus,
-                                    [analyte], win)
+            self.grads = calc_grads(x=self.Time, dat=self.focus,
+                                    keys=[analyte], win=win, win_mode=win_mode)
             self.grads_calced = True
 
-        below, above = filters.threshold(abs(self.grads[analyte]), threshold)
+        if absolute_gradient:
+            below, above = filters.threshold(abs(self.grads[analyte]), threshold)
+        else:
+            below, above = filters.threshold(self.grads[analyte], threshold)
 
         setn = self.filt.maxset + 1
 
@@ -1348,7 +1374,7 @@ class D(object):
                           focus_stage=focus_stage, err_envelope=err_envelope, ax=ax)
 
     @_log
-    def gplot(self, analytes=None, win=5, figsize=[10, 4],
+    def gplot(self, analytes=None, win=5, figsize=[10, 4], filt=False, recalc=False,
               ranges=False, focus_stage=None, ax=None):
         """
         Plot analytes gradients as a function of Time.
@@ -1370,8 +1396,8 @@ class D(object):
         figure, axis
         """
 
-        return plot.gplot(self=self, analytes=analytes, win=win, figsize=figsize,
-                          ranges=ranges, focus_stage=focus_stage, ax=ax)
+        return plot.gplot(self=self, analytes=analytes, win=win, figsize=figsize, filt=filt,
+                          ranges=ranges, focus_stage=focus_stage, ax=ax, recalc=recalc)
 
         # if type(analytes) is str:
         #     analytes = [analytes]
