@@ -1,3 +1,9 @@
+"""
+Plotting functions.
+
+(c) Oscar Branson : https://github.com/oscarbranson
+"""
+
 import itertools, re, warnings
 import numpy as np
 import matplotlib.pyplot as plt
@@ -9,7 +15,8 @@ from pandas import IndexSlice as idx
 
 from tqdm import tqdm
 
-from .helpers import fastgrad, fastsmooth, findmins, bool_2_indices, rangecalc, unitpicker, pretty_element, calc_grads
+from .helpers import fastgrad, fastsmooth, findmins, bool_2_indices, rangecalc, unitpicker, calc_grads
+from .analyte_names import pretty_element
 from .stat_fns import nominal_values, gauss, R2calc, unpack_uncertainties
 
 def calc_nrow(n, ncol):
@@ -55,17 +62,13 @@ def tplot(self, analytes=None, figsize=[10, 4], scale='log', filt=None,
         -------
         figure, axis
         """
-        if type(analytes) is str:
-            analytes = [analytes]
-        if analytes is None:
-            analytes = self.analytes
-
         if focus_stage is None:
             focus_stage = self.focus_stage
-        
-        # exclude internal standard from analytes
-        if focus_stage in ['ratios', 'calibrated']:
-            analytes = [a for a in analytes if a != self.internal_standard]
+
+        if analytes is None:
+            analytes = self.analytes        
+            if focus_stage in ['ratios', 'calibrated']:
+                analytes = self.analyte_ratios
 
         if ax is None:
             fig = plt.figure(figsize=figsize)
@@ -94,12 +97,12 @@ def tplot(self, analytes=None, figsize=[10, 4], scale='log', filt=None,
                     yerrf[~ind] = np.nan
                 if any(~ind):
                     ax.plot(x, y, color=self.cmap[a], alpha=.2, lw=0.6)
-                ax.plot(xf, yf, color=self.cmap[a], label=a)
+                ax.plot(xf, yf, color=self.cmap[a], label=pretty_element(a))
                 if err_envelope:
                     ax.fill_between(xf, yf - yerrf, yf + yerrf, color=self.cmap[a],
                                     alpha=0.2, zorder=-1)
             else:
-                ax.plot(x, y, color=self.cmap[a], label=a)
+                ax.plot(x, y, color=self.cmap[a], label=pretty_element(a))
                 if err_envelope:
                     ax.fill_between(x, y - yerr, y + yerr, color=self.cmap[a],
                                     alpha=0.2, zorder=-1)
@@ -154,10 +157,9 @@ def tplot(self, analytes=None, figsize=[10, 4], scale='log', filt=None,
         ud = {'rawdata': 'counts',
               'despiked': 'counts',
               'bkgsub': 'background corrected counts',
-              'ratios': 'counts/{:s} count',
-              'calibrated': 'mol/mol {:s}'}
-        if focus_stage in ['ratios', 'calibrated']:
-            ud[focus_stage] = ud[focus_stage].format(self.internal_standard)
+              'ratios': 'counts/count',
+              'calibrated': 'mol/mol',
+              'mass_fraction': 'Mass Fraction'}
         ax.set_ylabel(ud[focus_stage])
 
         # if interactive:
@@ -172,8 +174,8 @@ def tplot(self, analytes=None, figsize=[10, 4], scale='log', filt=None,
         if ret:
             return fig, ax
 
-def gplot(self, analytes=None, win=25, figsize=[10, 4],
-              ranges=False, focus_stage=None, ax=None, recalc=True):
+def gplot(self, analytes=None, win=25, figsize=[10, 4], filt=False,
+          ranges=False, focus_stage=None, ax=None, recalc=True):
         """
         Plot analytes gradients as a function of Time.
 
@@ -193,14 +195,16 @@ def gplot(self, analytes=None, win=25, figsize=[10, 4],
         -------
         figure, axis
         """
-
-        if type(analytes) is str:
-            analytes = [analytes]
-        if analytes is None:
-            analytes = self.analytes
-
         if focus_stage is None:
             focus_stage = self.focus_stage
+        
+        if isinstance(analytes, str):
+            analytes = [analytes]
+        elif analytes is None:
+            if self.grads_calced:
+                analytes = self.grads.keys()
+            else:
+                analytes = self.data[focus_stage].keys()
 
         if ax is None:
             fig = plt.figure(figsize=figsize)
@@ -216,7 +220,20 @@ def gplot(self, analytes=None, win=25, figsize=[10, 4],
             self.grads_calce = True
 
         for a in analytes:
-            ax.plot(x, self.grads[a], color=self.cmap[a], label=a)
+            y = self.grads[a]
+            if filt:
+                ind = self.filt.grab_filt(filt, a)
+                xf = x.copy()
+                yf = y.copy()
+                if any(~ind):
+                    xf[~ind] = np.nan
+                    yf[~ind] = np.nan
+                if any(~ind):
+                    ax.plot(x, y, color=self.cmap[a], alpha=.2, lw=0.6)
+                ax.plot(xf, yf, color=self.cmap[a], label=pretty_element(a))
+            else:
+                ax.plot(x, y, color=self.cmap[a], label=pretty_element(a))
+            # ax.plot(x, self.grads[a], color=self.cmap[a], label=pretty_element(a))
 
         if ranges:
             for lims in self.bkgrng:
@@ -235,10 +252,9 @@ def gplot(self, analytes=None, win=25, figsize=[10, 4],
         ud = {'rawdata': 'counts/s',
               'despiked': 'counts/s',
               'bkgsub': 'background corrected counts/s',
-              'ratios': 'counts/{:s} count/s',
-              'calibrated': 'mol/mol {:s}/s'}
-        if focus_stage in ['ratios', 'calibrated']:
-            ud[focus_stage] = ud[focus_stage].format(self.internal_standard)
+              'ratios': 'counts/count/s',
+              'calibrated': 'mol/mol/s',
+              'mass_fraction': 'Mass Fraction/s'}
         ax.set_ylabel(ud[focus_stage])
         # y tick format
 
@@ -288,7 +304,7 @@ def crossplot(dat, keys=None, lognorm=True, bins=25, figsize=(12, 12),
         figsize = [1.5 * numvar] * 2
     
     fig, axes = plt.subplots(nrows=numvar, ncols=numvar,
-                             figsize=(12, 12))
+                             figsize=figsize)
     fig.subplots_adjust(hspace=0.05, wspace=0.05)
 
     for ax in axes.flat:
@@ -689,14 +705,14 @@ def autorange_plot(t, sig, gwin=7, swin=None, win=30,
 
     return fig, axs
 
-def calibration_plot(self, analytes=None, datarange=True, loglog=False, ncol=3, srm_group=None, save=True):
+def calibration_plot(self, analyte_ratios=None, datarange=True, loglog=False, ncol=3, srm_group=None, percentile_data_cutoff=85, save=True):
     """
     Plot the calibration lines between measured and known SRM values.
 
     Parameters
     ----------
-    analytes : optional, array_like or str
-        The analyte(s) to plot. Defaults to all analytes.
+    analyte_ratios : optional, array_like or str
+        The analyte ratio(s) to plot. Defaults to all analyte ratios.
     datarange : boolean
         Whether or not to show the distribution of the measured data
         alongside the calibration curve.
@@ -705,17 +721,25 @@ def calibration_plot(self, analytes=None, datarange=True, loglog=False, ncol=3,
         useful if you have two low standards very close together,
         and want to check whether your data are between them, or
         below them.
+    ncol : int
+        The number of columns in the plot
+    srm_group : int
+        Which groups of SRMs to plot in the analysis.
+    percentile_data_cutoff : float
+        The upper percentile of data to display in the histogram.
 
     Returns
     -------
     (fig, axes)
     """
 
-    if isinstance(analytes, str):
-        analytes = [analytes]
+    if isinstance(analyte_ratios, str):
+        analyte_ratios = [analyte_ratios]
 
-    if analytes is None:
-        analytes = [a for a in self.analytes if self.internal_standard not in a]
+    if analyte_ratios is None:
+        # analytes = self._calib_analytes
+        analyte_ratios = self.analytes_sorted(self._srm_id_analyte_ratios)
+        # analytes = self.analytes_sorted(self.analytes.difference([self.internal_standard]))
 
     if srm_group is not None:
         srm_groups = {int(g): t for g, t in self.stdtab.loc[:, ['group', 'gTime']].values}
@@ -730,7 +754,7 @@ def calibration_plot(self, analytes=None, datarange=True, loglog=False, ncol=3,
         gTime = None
 
     ncol = int(ncol)
-    n = len(analytes)
+    n = len(analyte_ratios)
     nrow = calc_nrow(n + 1, ncol)
 
     axes = []
@@ -746,7 +770,8 @@ def calibration_plot(self, analytes=None, datarange=True, loglog=False, ncol=3,
 
     mdict = self.srm_mdict
 
-    for g, a in zip(gs, analytes):
+    for g, a in zip(gs, analyte_ratios):
+        num, denom = a.split('_')
         if not datarange:
             ax = fig.add_axes(g.get_position(fig))
             axes.append((ax,))
@@ -760,14 +785,14 @@ def calibration_plot(self, analytes=None, datarange=True, loglog=False, ncol=3,
             axes.append((ax, axh))
         
         if gTime is None:
-            sub = idx[a]
+            sub = idx[:,:]
         else:
-            sub = idx[a, :, :, gTime]
-        x = self.srmtabs.loc[sub, 'meas_mean'].values
-        xe = self.srmtabs.loc[sub, 'meas_err'].values
-        y = self.srmtabs.loc[sub, 'srm_mean'].values
-        ye = self.srmtabs.loc[sub, 'srm_err'].values
-        srm = self.srmtabs.loc[sub].index.get_level_values('SRM')
+            sub = idx[gTime, :]
+        x = self.caltab.loc[sub, (a, 'meas_mean')].values
+        xe = self.caltab.loc[sub, (a, 'meas_err')].values
+        y = self.caltab.loc[sub, (a, 'srm_mean')].values
+        ye = self.caltab.loc[sub, (a, 'srm_err')].values
+        srm = self.caltab.loc[sub, ('SRM')]
         
         # plot calibration data
         for s, m in mdict.items():
@@ -796,8 +821,8 @@ def calibration_plot(self, analytes=None, datarange=True, loglog=False, ncol=3,
                 ylim = [0, ymax * 1.05]
 
         else:
-            xd = self.srmtabs.loc[a, 'meas_mean'][self.srmtabs.loc[a, 'meas_mean'] > 0].values
-            yd = self.srmtabs.loc[a, 'srm_mean'][self.srmtabs.loc[a, 'srm_mean'] > 0].values
+            xd = self.caltab.loc[:, (a, 'meas_mean')][self.caltab.loc[:, (a, 'meas_mean')] > 0].values
+            yd = self.caltab.loc[:, (a, 'srm_mean')][self.caltab.loc[:, (a, 'srm_mean')] > 0].values
 
             xlim = [10**np.floor(np.log10(np.nanmin(xd))),
                     10**np.ceil(np.log10(np.nanmax(xd)))]
@@ -843,8 +868,8 @@ def calibration_plot(self, analytes=None, datarange=True, loglog=False, ncol=3,
             c = np.nanmean(coefs['c'])
             c_nom = nominal_values(c)
             # calculate R2
-            ym = self.srmtabs.loc[a, 'meas_mean'] * m_nom + c_nom
-            R2 = R2calc(self.srmtabs.loc[a, 'srm_mean'], ym, force_zero=False)
+            ym = self.caltab.loc[:, (a, 'meas_mean')] * m_nom + c_nom
+            R2 = R2calc(self.caltab.loc[:, (a, 'srm_mean')], ym, force_zero=False)
             # generate line and label
             line = x * m_nom + c_nom
             label = 'y = {:.2e} x'.format(m)
@@ -854,8 +879,8 @@ def calibration_plot(self, analytes=None, datarange=True, loglog=False, ncol=3,
                 label += '\n {:.2e}'.format(c)
         else:
             # calculate R2
-            ym = self.srmtabs.loc[a, 'meas_mean'] * m_nom
-            R2 = R2calc(self.srmtabs.loc[a, 'srm_mean'], ym, force_zero=True)
+            ym = self.caltab.loc[:, (a, 'meas_mean')] * m_nom
+            R2 = R2calc(self.caltab.loc[:, (a, 'srm_mean')], ym, force_zero=True)
             # generate line and label
             line = x * m_nom
             label = 'y = {:.2e} x'.format(m)
@@ -871,8 +896,8 @@ def calibration_plot(self, analytes=None, datarange=True, loglog=False, ncol=3,
 
         ax.text(.05, .95, pretty_element(a), transform=ax.transAxes,
                 weight='bold', va='top', ha='left', size=12)
-        ax.set_xlabel('counts/counts ' + self.internal_standard)
-        ax.set_ylabel('mol/mol ' + self.internal_standard)
+        ax.set_xlabel('counts/counts')
+        ax.set_ylabel('mol/mol')
         # write calibration equation on graph happens after data distribution
 
         # plot data distribution historgram alongside calibration plot
@@ -888,15 +913,14 @@ def calibration_plot(self, analytes=None, datarange=True, loglog=False, ncol=3,
                     ylim[0] = 10**np.floor(np.log10(np.nanmin(mmeas)))
                 else:
                     ylim[0] = 0
-                ax.set_ylim(ylim)
 
-            m95 = np.percentile(meas[~np.isnan(meas)], 95) * 1.05
-            if m95 > ylim[1]:
+            mquant = np.percentile(meas[~np.isnan(meas)], percentile_data_cutoff) * 1.05
+            if mquant > ylim[1]:
                 if loglog:
-                    ylim[1] = 10**np.ceil(np.log10(m95))
+                    ylim[1] = 10**np.ceil(np.log10(mquant))
                 else:
-                    ylim[1] = m95
-
+                    ylim[1] = mquant
+            
             # hist
             if loglog:
                 bins = np.logspace(*np.log10(ylim), 30)
@@ -1020,16 +1044,12 @@ def filter_report(Data, filt=None, analytes=None, savedir=None, nbin=5):
     (fig, axes)
     """
     if filt is None or filt == 'all':
-        sets = Data.filt.sets
+        sets = Data.filt.filter_table
     else:
-        sets = {k: v for k, v in Data.filt.sets.items() if any(filt in f for f in v)}
-
-    regex = re.compile('^([0-9]+)_([A-Za-z0-9-]+)_'
-                    '([A-Za-z0-9-]+)[_$]?'
-                    '([a-z0-9]+)?')
+        sets = Data.filt.filter_table.loc[idx[:, [i for i in Data.filt.filter_table.index.levels[1] if filt in i]], :]
 
     cm = plt.cm.get_cmap('Spectral')
-    ngrps = len(sets)
+    ngrps = sets.shape[0]
 
     if analytes is None:
         analytes = Data.analytes
@@ -1038,118 +1058,84 @@ def filter_report(Data, filt=None, analytes=None, savedir=None, nbin=5):
 
     axes = []
     for analyte in analytes:
-        if analyte != Data.internal_standard:
-            fig = plt.figure()
+        fig = plt.figure()
 
-            for i in sorted(sets.keys()):
-                filts = sets[i]
-                nfilts = np.array([re.match(regex, f).groups() for f in filts])
-                fgnames = np.array(['_'.join(a) for a in nfilts[:, 1:3]])
-                fgrp = np.unique(fgnames)[0]
+        for i in sets.index.levels[0]:
+            filts = sets.loc[i]
+            nfilts = np.array([f.split('_') for f in filts.index])
+            fgnames = np.array(['_'.join(a) for a in nfilts[:, (0,-2)]])
+            fgrp = np.unique(fgnames)[0]
+            
+            if 'DBSCAN' in fgrp:
+                warnings.warn('filter_report is not implemented for DBSCAN')
+                continue
 
-                fig.set_size_inches(10, 3.5 * ngrps)
-                h = .8 / ngrps
+            fig.set_size_inches(10, 3.5 * ngrps)
+            h = .8 / ngrps
 
-                y = nominal_values(Data.focus[analyte])
-                yh = y[~np.isnan(y)]
+            y = nominal_values(Data.focus[analyte])
+            yh = y[~np.isnan(y)]
 
-                m, u = unitpicker(np.nanmax(y),
-                                denominator=Data.internal_standard,
-                                focus_stage=Data.focus_stage)
+            m, u = unitpicker(np.nanmax(y),
+                            denominator=Data.internal_standard,
+                            focus_stage=Data.focus_stage)
 
-                axs = tax, hax = (fig.add_axes([.1, .9 - (i + 1) * h, .6, h * .98]),
-                                fig.add_axes([.7, .9 - (i + 1) * h, .2, h * .98]))
-                axes.append(axs)
+            axs = tax, hax = (fig.add_axes([.1, .9 - (i + 1) * h, .6, h * .98]),
+                            fig.add_axes([.7, .9 - (i + 1) * h, .2, h * .98]))
+            axes.append(axs)
 
-                # get variables
-                fg = sets[i]
-                cs = cm(np.linspace(0, 1, len(fg)))
-                fn = ['_'.join(x) for x in nfilts[:, (0, 3)]]
-                an = nfilts[:, 0]
-                bins = np.linspace(np.nanmin(y), np.nanmax(y), len(yh) // nbin) * m
+            # get variables
+            fg = sets.index.get_level_values(1)  # filter names
+            components = Data.filt.filter_components.loc[:, i]  # component dataframe
+            cs = cm(np.linspace(0, 1, len(fg)))
+            fn = ['_'.join(x) for x in nfilts[:, (0, -1)]]
+            an = nfilts[:, 0]
+            bins = np.linspace(np.nanmin(y), np.nanmax(y), len(yh) // nbin) * m
 
-                if 'DBSCAN' in fgrp:
-                    # determine data filters
-                    core_ind = Data.filt.components[[f for f in fg
-                                                    if 'core' in f][0]]
-                    other = np.array([('noise' not in f) & ('core' not in f)
-                                    for f in fg])
-                    tfg = fg[other]
-                    tfn = fn[other]
-                    tcs = cm(np.linspace(0, 1, len(tfg)))
+            # plot all data
+            tax.scatter(Data.Time, m * y, color='k', alpha=0.2, lw=0.1,
+                        s=20, label='excl')
+            hax.hist(m * yh, bins, alpha=0.2, orientation='horizontal',
+                     color='k', lw=0)
 
-                    # plot all data
-                    hax.hist(m * yh, bins, alpha=0.2, orientation='horizontal',
-                            color='k', lw=0)
-                    # legend markers for core/member
-                    tax.scatter([], [], s=20, label='core', color='w', lw=0.5, edgecolor='k')
-                    tax.scatter([], [], s=7.5, label='member', color='w', lw=0.5, edgecolor='k')
-                    # plot noise
-                    try:
-                        noise_ind = Data.filt.components[[f for f in fg
-                                                        if 'noise' in f][0]]
-                        tax.scatter(Data.Time[noise_ind], m * y[noise_ind],
-                                    lw=1, color='k', s=10, marker='x',
-                                    label='noise', alpha=0.6)
-                    except:
-                        pass
+            # plot filtered data
+            for f, c, lab in zip(fg, cs, fn):
+                ind = components[f]
+                tax.scatter(Data.Time[ind], m * y[ind],
+                            edgecolor=(0,0,0,0), color=c, s=15, label=lab)
+                hax.hist(m * y[ind][~np.isnan(y[ind])], bins, color=c, lw=0.1,
+                        orientation='horizontal', alpha=0.6)
 
-                    # plot filtered data
-                    for f, c, lab in zip(tfg, tcs, tfn):
-                        ind = Data.filt.components[f]
-                        tax.scatter(Data.Time[~core_ind & ind],
-                                    m * y[~core_ind & ind], lw=.5, color=c, s=5, edgecolor='k')
-                        tax.scatter(Data.Time[core_ind & ind],
-                                    m * y[core_ind & ind], lw=.5, color=c, s=15, edgecolor='k',
-                                    label=lab)
-                        hax.hist(m * y[ind][~np.isnan(y[ind])], bins, color=c, lw=0.1,
-                                orientation='horizontal', alpha=0.6)
+            if 'thresh' in fgrp and analyte in fgrp:
+                tax.axhline(Data.filt.params[fg[0]]['threshold'] * m,
+                            ls='dashed', zorder=-2, alpha=0.5, color='k')
+                hax.axhline(Data.filt.params[fg[0]]['threshold'] * m,
+                            ls='dashed', zorder=-2, alpha=0.5, color='k')
 
-                else:
-                    # plot all data
-                    tax.scatter(Data.Time, m * y, color='k', alpha=0.2, lw=0.1,
-                                s=20, label='excl')
-                    hax.hist(m * yh, bins, alpha=0.2, orientation='horizontal',
-                             color='k', lw=0)
+            # formatting
+            for ax in axs:
+                mn = np.nanmin(y) * m
+                mx = np.nanmax(y) * m
+                rn = mx - mn
+                ax.set_ylim(mn - .05 * rn, mx + 0.05 * rn)
 
-                    # plot filtered data
-                    for f, c, lab in zip(fg, cs, fn):
-                        ind = Data.filt.components[f]
-                        tax.scatter(Data.Time[ind], m * y[ind],
-                                    edgecolor=(0,0,0,0), color=c, s=15, label=lab)
-                        hax.hist(m * y[ind][~np.isnan(y[ind])], bins, color=c, lw=0.1,
-                                orientation='horizontal', alpha=0.6)
+            # legend
+            hn, la = tax.get_legend_handles_labels()
+            hax.legend(hn, la, loc='upper right', scatterpoints=1)
 
-                if 'thresh' in fgrp and analyte in fgrp:
-                    tax.axhline(Data.filt.params[fg[0]]['threshold'] * m,
-                                ls='dashed', zorder=-2, alpha=0.5, color='k')
-                    hax.axhline(Data.filt.params[fg[0]]['threshold'] * m,
-                                ls='dashed', zorder=-2, alpha=0.5, color='k')
+            tax.text(.02, .98, Data.sample + ': ' + fgrp, size=12,
+                    weight='bold', ha='left', va='top',
+                    transform=tax.transAxes)
+            tax.set_ylabel(pretty_element(analyte) + ' (' + u + ')')
+            tax.set_xticks(tax.get_xticks()[:-1])
+            hax.set_yticklabels([])
 
-                # formatting
-                for ax in axs:
-                    mn = np.nanmin(y) * m
-                    mx = np.nanmax(y) * m
-                    rn = mx - mn
-                    ax.set_ylim(mn - .05 * rn, mx + 0.05 * rn)
-
-                # legend
-                hn, la = tax.get_legend_handles_labels()
-                hax.legend(hn, la, loc='upper right', scatterpoints=1)
-
-                tax.text(.02, .98, Data.sample + ': ' + fgrp, size=12,
-                        weight='bold', ha='left', va='top',
-                        transform=tax.transAxes)
-                tax.set_ylabel(pretty_element(analyte) + ' (' + u + ')')
-                tax.set_xticks(tax.get_xticks()[:-1])
-                hax.set_yticklabels([])
-
-                if i < ngrps - 1:
-                    tax.set_xticklabels([])
-                    hax.set_xticklabels([])
-                else:
-                    tax.set_xlabel('Time (s)')
-                    hax.set_xlabel('n')
+            if i < ngrps - 1:
+                tax.set_xticklabels([])
+                hax.set_xticklabels([])
+            else:
+                tax.set_xlabel('Time (s)')
+                hax.set_xlabel('n')
 
         if isinstance(savedir, str):
             fig.savefig(savedir + '/' + Data.sample + '_' +
@@ -1158,6 +1144,44 @@ def filter_report(Data, filt=None, analytes=None, savedir=None, nbin=5):
 
     return fig, axes
 
+# Old DBSCAN code
+# if 'DBSCAN' in fgrp:
+#                     # determine data filters
+#                     core_ind = components[[f for f in fg
+#                                                     if 'core' in f][0]]
+#                     other = np.array([('noise' not in f) & ('core' not in f)
+#                                     for f in fg])
+#                     tfg = fg[other]
+#                     tfn = fn[other]
+#                     tcs = cm(np.linspace(0, 1, len(tfg)))
+
+#                     # plot all data
+#                     hax.hist(m * yh, bins, alpha=0.2, orientation='horizontal',
+#                             color='k', lw=0)
+#                     # legend markers for core/member
+#                     tax.scatter([], [], s=20, label='core', color='w', lw=0.5, edgecolor='k')
+#                     tax.scatter([], [], s=7.5, label='member', color='w', lw=0.5, edgecolor='k')
+#                     # plot noise
+#                     try:
+#                         noise_ind = components[[f for f in fg
+#                                                         if 'noise' in f][0]]
+#                         tax.scatter(Data.Time[noise_ind], m * y[noise_ind],
+#                                     lw=1, color='k', s=10, marker='x',
+#                                     label='noise', alpha=0.6)
+#                     except:
+#                         pass
+
+#                     # plot filtered data
+#                     for f, c, lab in zip(tfg, tcs, tfn):
+#                         ind = components[f]
+#                         tax.scatter(Data.Time[~core_ind & ind],
+#                                     m * y[~core_ind & ind], lw=.5, color=c, s=5, edgecolor='k')
+#                         tax.scatter(Data.Time[core_ind & ind],
+#                                     m * y[core_ind & ind], lw=.5, color=c, s=15, edgecolor='k',
+#                                     label=lab)
+#                         hax.hist(m * y[ind][~np.isnan(y[ind])], bins, color=c, lw=0.1,
+#                                 orientation='horizontal', alpha=0.6)
+
 def correlation_plot(self, corr=None):
     if len(self.correlations) == 0:
         raise ValueError("No calculated correlations. Run threshold_correlation first.")
