@@ -8,6 +8,7 @@ import os
 import json
 import datetime
 import dateutil
+import textwrap
 import numpy as np
 import pandas as pd
 import pkg_resources as pkgrs
@@ -19,6 +20,8 @@ from ..helpers.io import read_dataformat
 
 import matplotlib.pyplot as plt
 
+textwidth = 70  # characters, for printing
+
 def by_regex(file, outdir=None, split_pattern=None, global_header_rows=0, fname_pattern=None, trim_tail_lines=0, trim_head_lines=0):
     """
     Split one long analysis file into multiple smaller ones.
@@ -123,6 +126,12 @@ def long_file(data_file, dataformat, sample_list, analyte='total_counts', savedi
         A valid dataformat dict. See online documentation for more details.
     sample_list : array-like
         A list of strings that will be used to name the individual files.
+        One sample name can contain a 'wildcard' character '+' or '*'. If
+        we find more ablations than the number of names in sample_list, we'll
+        expand this wildcard to name each unlabelled ablation. If the wildcard
+        is '+' the abltations are given unique numbered names and split up into
+        separate files, whereas for '*' the ablations are given the same and
+        saved into a single file. One *one* sample name can contain a wildcard.
     analyte : str
         The analyte that autorange uses to identify ablations. Can be any valid
         analyte in the data. Defaults to 'total_counts'.
@@ -144,7 +153,29 @@ def long_file(data_file, dataformat, sample_list, analyte='total_counts', savedi
             raise ValueError('File {} not found.')
     else:
         sample_list = np.asanyarray(sample_list)
-        
+
+    # detect wildcard samples
+    # * = multiple analyses to be numbered
+    # + = multiple analyses to be combined
+    mode = 'strict'
+    wilds = []
+    for s in sample_list:
+        if '*' in s:
+            mode = '*'
+            wilds.append(s)
+        if '+' in s:
+            mode = '+'
+            wilds.append(s)
+    if len(wilds) > 1:
+        errmsg = (
+            ["More than one sample name contains a wildcard:"] + 
+            [f"   {w}" for w in wilds] + 
+            ["I don't know how to cope with this..."])
+        raise ValueError('\n'.join(errmsg))
+    if len(wilds) == 1:
+        wildind = np.argwhere(sample_list == wilds[0]).item()
+        wildsample = wilds[0].replace('+','').replace('*','')
+
     if srm_id is not None:
         srm_replace = []
         for s in sample_list:
@@ -152,11 +183,11 @@ def long_file(data_file, dataformat, sample_list, analyte='total_counts', savedi
                 s = srm_id
             srm_replace.append(s)
         sample_list = srm_replace
-    
+
     dataformat = read_dataformat(dataformat, silent=False)
-                    
+
     _, _, dat, meta = read_data(data_file, dataformat=dataformat, name_mode='file')
-    
+
     if 'date' in meta:
         d = dateutil.parser.parse(meta['date'])
     else:
@@ -170,16 +201,40 @@ def long_file(data_file, dataformat, sample_list, analyte='total_counts', savedi
     else:
         valid = list(dat['rawdata'].keys()) + ['total_counts']
         raise ValueError("'{}' is not a valid analyte. Please use one of:\n  {}".format(analyte, valid))
-    
+
     # autorange
     bkg, sig, _, _ = autorange(dat['Time'], y_data, **autorange_args)
-    
+
     ns = np.zeros(sig.size)
     ns[sig] = np.cumsum((sig ^ np.roll(sig, 1)) & sig)[sig]
     n = int(max(ns))
 
     nsamples = len(sample_list)
 
+    # deal with wildcards
+    if nsamples <= n and mode != 'strict':
+        msg = f"There are more ablations than samples in list, but you've given wildcard '{mode}' for sample '{wildsample}'..."
+        print('\n'.join(textwrap.wrap(msg, textwidth)))
+
+        pre = sample_list[:wildind]
+        post = sample_list[wildind + 1:]
+        nnew = n - nsamples + 1
+        
+        # deal with '*' wildcard
+        if mode == '+':
+            newnames = [f"{wildsample}_{n}" for n in range(nnew)]
+            msg = f'  -> These {nnew} ablations will be prepended with consecutive numbers and split into separate files.'
+
+        # deal with '+' wildcard
+        if mode == '*':
+            newnames = [f"{wildsample}" for n in range(nnew)]
+            msg = f'  -> These {nnew} ablations will be given the same name and combined into a single file.'
+        
+        print('\n     '.join(textwrap.wrap(msg, textwidth - 5)))
+
+        sample_list = np.concatenate([pre, newnames, post])
+        nsamples = len(sample_list)
+
     if nsamples != n:
         print('Number of samples in list ({}) does not match number of ablations ({}).'.format(nsamples, n))
         if nsamples < n:
